/**
 * Copyright (c) 2013 Stefan Marr, stefan.marr@vub.ac.be
 * Software Architecture Group, Hasso Plattner Institute, Potsdam, Germany
 * http://www.hpi.uni-potsdam.de/swa/
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package som.interpreter;


/**
 * The FrameOnStackMarker is a marker to represent the identity of frames, and
 * to keep track on the status of whether the frame is still on the stack.
 * Currently it is used to implement non-local returns by marking
 * stack frames with it, and checking for the marker during unwinding.
 *
 * @author Stefan Marr
 */
public final class FrameOnStackMarker {
  private boolean isOnStack;

  public FrameOnStackMarker() {
    isOnStack = true;
  }

  public void frameNoLongerOnStack() {
    isOnStack = false;
  }

  public boolean isOnStack() {
    return isOnStack;
  }
}
