## This file defines common functions used for data processing.

load_data_file <- function (file, row_names) {
  if (missing(row_names)) {
    # these row names are hard coded and might not be applicable
    # names like this are generated by ReBench
    row_names <- c("TimeStamp", "Value", "Unit", "Criterion", "Benchmark",
                   "VM", "Suite", "Extra", "Warmup", "Cores", "InputSize",
                   "Var")
  }
  
  bench <- read.table(file, sep="\t", header=FALSE, col.names=row_names,
                            fill=TRUE) # rbind()
  bench$rid = seq_len(nrow(bench))
  bench
}

prepare_vm_names <- function(data) {
  # Reorder
  data$VM <- factor(data$VM, c("Java", "PyPy",
                               "RTruffleSOM-jit",
                               "RTruffleSOM-OMOP-jit",
                               "TruffleSOM-graal",
                               "TruffleSOM-OMOP-graal",
                               "TruffleSOM-OMOP-graal-no-stamp",
                               "TruffleSOM-OMOP-graal-old-splitting",
                               "TruffleSOM-graal-no-split",
                               "TruffleSOM-graal-split-extra",
                               "TruffleSOM-graal-old-splitting",
                               "JRuby-meta-uncached",
                               "JRuby",
                               "Ruby21",
                               "SOMpp"))
  
  name_map <-     list("Java"                  = "Java",
                       "PyPy"                  = "PyPy",
                       "RTruffleSOM-jit"       = "RTruffleSOM",
                       "RTruffleSOM-OMOP-jit"       = "RTruffleSOM (OMOP)",
                       
                       "TruffleSOM-OMOP-graal"          = "TruffleSOM.ns (OMOP)",
                       "TruffleSOM-OMOP-graal-old-splitting" = "TruffleSOM.os (OMOP)",
                       "TruffleSOM-OMOP-graal-no-stamp"      = "TruffleSOM.nse (OMOP)",
                       "TruffleSOM-graal"               = "TruffleSOM.ns",
                       "TruffleSOM-graal-split-extra"   = "TruffleSOM.nse",
                       "TruffleSOM-graal-no-split"      = "TruffleSOM.wos",
                       "TruffleSOM-graal-old-splitting" = "TruffleSOM.os",
                       "JRuby"               = "JRuby",
                       "JRuby-meta-uncached" = "JRuby-meta-uncached",
                       "Ruby21"              = "Ruby",
                       "SOMpp"               = "SOM++")
  # Rename
  levels(data$VM)  <- map_names(
    levels(data$VM),
    name_map)
  data
}

map_names <- function(old_names, name_map) {
  for (i in 1:length(old_names)) {
    old_name <- old_names[[i]]
    if (!is.null(name_map[[old_name]])) {
      old_names[i] <- name_map[[old_name]]
    }
  }
  old_names
}
